import 'dart:async';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:idkit/src/function/list/func_num_list.dart';
import 'package:idkit/src/function/num/func_num.dart';

class IDKitGestureRecognizer extends StatefulWidget {
  const IDKitGestureRecognizer({
    super.key,
    this.child,
    this.onTap,
    this.onStart,
    this.onUpdate,
    this.onEnd,
    this.fingerCount,
    this.minPressure = 0,
    this.maxPressure = 1,
    this.forcePressOnStart,
    this.forcePressOnUpdate,
    this.forcePressOnEnd,
    this.forcePressOnCancel,
    this.duration = const Duration(milliseconds: 300),
  });

  /// Callback for click event.
  final Function(IDKitTapDetails details)? onTap;

  /// Gesture zoom, move start method call.
  final Function(IDKitStartDetails details)? onStart;

  /// Gesture zoom, mobile update method call.
  final Function(IDKitUpdateDetails details)? onUpdate;

  /// Gesture zooming and moving end method calls.
  final Function(IDKitEndDetails details)? onEnd;

  /// Callback for counting the number of fingers on the screen, one does not count.
  final Function(int)? fingerCount;

  /// Set the maximum and minimum range of pressure sensitivity
  final double minPressure;
  final double maxPressure;

  /// Screen pressure monitoring starts.
  final Function(PointerDownEvent event)? forcePressOnStart;

  /// On-Screen Pressure Monitoring Update.
  final Function(PointerMoveEvent event)? forcePressOnUpdate;

  /// Screen pressure monitoring ends.
  final Function(PointerUpEvent event)? forcePressOnEnd;

  /// End of pressure for touchscreen devices.
  final Function(PointerCancelEvent event)? forcePressOnCancel;

  /// finger racing time.
  final Duration duration;

  /// Sub widgets of the Gestures widget.
  final Widget? child;

  @override
  State<IDKitGestureRecognizer> createState() => _IDKitGestureRecognizerState();
}

class _IDKitGestureRecognizerState extends State<IDKitGestureRecognizer> {
  /// Record of fingers
  int count = 0;

  /// Whether to trigger the record of the finger counting end method
  bool isOnSerialTapUp = false;

  /// Whether to trigger the recording of the zoom start method
  bool isOnStart = false;

  /// Whether to trigger the recording of zoom end method
  bool isOnEnd = false;

  /// Pressure on touchscreen devices
  num _pressure = 0;

  /// Types of touch screen devices.
  PointerDeviceKind? _kind;

  /// Record initial point.
  Offset initialPoint = Offset.zero;

  /// Single Point Pressure Recording
  num singlePointPressure = 0;

  /// 记录单点压力集合
  List<double> pressureList = [];

  @override
  Widget build(BuildContext context) {
    return Listener(
      onPointerDown: (PointerDownEvent event) {
        _kind = event.kind;
        pressureList.clear();
        _pressure = event.pressure.range(
          min: widget.minPressure,
          max: widget.maxPressure,
        );
        singlePointPressure = _pressure;
        widget.forcePressOnStart?.call(event);
      },
      onPointerMove: (PointerMoveEvent event) {
        _kind = event.kind;
        _pressure = event.pressure.range(
          min: widget.minPressure,
          max: widget.maxPressure,
        );
        pressureList.add(_pressure * 1.0);
        widget.forcePressOnUpdate?.call(event);
      },
      onPointerUp: (PointerUpEvent event) {
        _kind = event.kind;
        _pressure = event.pressure.range(
          min: widget.minPressure,
          max: widget.maxPressure,
        );
        singlePointPressure = pressureList.average;
        pressureList.clear();
        widget.forcePressOnEnd?.call(event);
      },
      child: RawGestureDetector(
        gestures: {
          TapGestureRecognizer: GestureRecognizerFactoryWithHandlers<TapGestureRecognizer>(
            () => TapGestureRecognizer(),
            (instance) {
              // instance.onTap = () {};
            },
          ),
          ScaleGestureRecognizer: GestureRecognizerFactoryWithHandlers<ScaleGestureRecognizer>(
            () => ScaleGestureRecognizer(),
            (instance) {
              instance.onStart = (ScaleStartDetails details) {
                if (!isOnSerialTapUp) count = 0;
                if (isOnEnd) return;
                if (!isOnStart) isOnStart = true;
                final IDKitStartDetails sDetails = IDKitStartDetails.pressure(_pressure, _kind, initialPoint, details);
                widget.onStart?.call(sDetails);
              };
              instance.onUpdate = (ScaleUpdateDetails details) {
                if (isOnEnd) return;
                final IDKitUpdateDetails sDetails = IDKitUpdateDetails.pressure(_pressure, _kind, details);
                widget.onUpdate?.call(sDetails);
              };
              instance.onEnd = (ScaleEndDetails details) {
                if (isOnEnd) return;
                isOnEnd = true;
                Timer.periodic(widget.duration, (timer) {
                  isOnEnd = false;
                  timer.cancel();
                });
                if (isOnStart) isOnStart = false;
                initialPoint = Offset.zero;
                final IDKitEndDetails sDetails = IDKitEndDetails.pressure(_pressure, _kind, details);
                widget.onEnd?.call(sDetails);
              };
            },
          ),
          SerialTapGestureRecognizer: GestureRecognizerFactoryWithHandlers<SerialTapGestureRecognizer>(
            () => SerialTapGestureRecognizer(),
            (SerialTapGestureRecognizer instance) {
              instance.onSerialTapDown = (details) {
                initialPoint = details.localPosition;
                isOnSerialTapUp = false;
                count++;
              };
              instance.onSerialTapUp = (SerialTapUpDetails details) {
                isOnSerialTapUp = true;
                if (count > 1) {
                  widget.fingerCount?.call(count);
                } else {
                  final IDKitTapDetails tapDetails = IDKitTapDetails.pressure(singlePointPressure, details);
                  widget.onTap?.call(tapDetails);
                }
                initialPoint = Offset.zero;
                count = 0;
              };
            },
          ),
        },
        child: widget.child,
      ),
    );
  }
}

/// Single point gesture
class IDKitTapDetails extends SerialTapUpDetails {
  IDKitTapDetails({
    this.pressure = 0,
    Offset globalPosition = Offset.zero,
    Offset? localPosition,
    PointerDeviceKind? kind,
    int count = 1,
  }) : super(
          globalPosition: globalPosition,
          localPosition: localPosition,
          kind: kind,
          count: count,
        );

  /// Finger pressure on the screen.
  final num pressure;

  factory IDKitTapDetails.pressure(num pressure, SerialTapUpDetails details) {
    return IDKitTapDetails(
      pressure: pressure,
      globalPosition: details.globalPosition,
      localPosition: details.localPosition,
      kind: details.kind,
      count: details.count,
    );
  }
}

/// Zoom and move start gesture information
class IDKitStartDetails extends ScaleStartDetails {
  IDKitStartDetails({
    this.pressure = 0,
    this.kind,
    this.initialPoint = Offset.zero,
    Offset focalPoint = Offset.zero,
    Offset? localFocalPoint,
    int pointerCount = 0,
  }) : super(
          focalPoint: focalPoint,
          localFocalPoint: localFocalPoint,
          pointerCount: pointerCount,
        );

  /// Finger pressure on the screen.
  final num pressure;

  /// Types of touch screen devices.
  final PointerDeviceKind? kind;

  /// Record initial point.
  final Offset initialPoint;

  /// Zoom, Rotate, Move Gesture Details
  factory IDKitStartDetails.pressure(
    num pressure,
    PointerDeviceKind? kind,
    Offset initialPoint,
    ScaleStartDetails details,
  ) {
    return IDKitStartDetails(
      pressure: pressure,
      kind: kind,
      initialPoint: initialPoint,
      focalPoint: details.focalPoint,
      localFocalPoint: details.localFocalPoint,
      pointerCount: details.pointerCount,
    );
  }
}

/// Zoom and move update gesture information
class IDKitUpdateDetails extends ScaleUpdateDetails {
  IDKitUpdateDetails({
    this.pressure = 0,
    this.kind,
    Offset focalPoint = Offset.zero,
    Offset? localFocalPoint,
    double scale = 1.0,
    double horizontalScale = 1.0,
    double verticalScale = 1.0,
    double rotation = 0.0,
    int pointerCount = 0,
    Offset focalPointDelta = Offset.zero,
  }) : super(
          focalPoint: focalPoint,
          localFocalPoint: localFocalPoint,
          scale: scale,
          horizontalScale: horizontalScale,
          verticalScale: verticalScale,
          rotation: rotation,
          pointerCount: pointerCount,
          focalPointDelta: focalPointDelta,
        );

  /// Finger pressure on the screen.
  final num pressure;

  /// Types of touch screen devices.
  final PointerDeviceKind? kind;

  factory IDKitUpdateDetails.pressure(
    num pressure,
    PointerDeviceKind? kind,
    ScaleUpdateDetails details,
  ) {
    return IDKitUpdateDetails(
      pressure: pressure,
      kind: kind,
      focalPoint: details.focalPoint,
      localFocalPoint: details.localFocalPoint,
      scale: details.scale,
      horizontalScale: details.horizontalScale,
      verticalScale: details.verticalScale,
      rotation: details.rotation,
      pointerCount: details.pointerCount,
      focalPointDelta: details.focalPointDelta,
    );
  }
}

/// Zoom and move end gesture information
class IDKitEndDetails extends ScaleEndDetails {
  IDKitEndDetails({
    this.pressure = 0,
    this.kind,
    Velocity velocity = Velocity.zero,
    int pointerCount = 0,
  }) : super(
          velocity: velocity,
          pointerCount: pointerCount,
        );

  /// Finger pressure on the screen.
  final num pressure;

  /// Types of touch screen devices.
  final PointerDeviceKind? kind;

  factory IDKitEndDetails.pressure(
    num pressure,
    PointerDeviceKind? kind,
    ScaleEndDetails details,
  ) {
    return IDKitEndDetails(
      pressure: pressure,
      kind: kind,
      velocity: details.velocity,
      pointerCount: details.pointerCount,
    );
  }
}
